WebSocket

Struct WebSocket 

pub struct WebSocket<H> { /* private fields */ }
The trillium handler. See crate-level docs for example usage.

Implementations

Source

impl<T> WebSocket<JsonHandler<T>>

pub fn new_json(handler: T) -> Self

Build a new trillium WebSocket handler from the provided JsonWebSocketHandler

Source

impl<H> WebSocket<H>

pub fn new(handler: H) -> Self

Build a new WebSocket with an async handler function that receives a WebSocketConn

pub fn with_protocols(self, protocols: &[&str]) -> Self

protocols is a sequence of known protocols. On successful handshake, the returned response headers contain the first protocol in this list which the server also knows.

pub fn with_protocol_config(self, config: WebSocketConfig) -> Self

configure the websocket protocol

pub fn required(self) -> Self

configure this handler to halt and send back a 426 Upgrade Required if a websocket cannot be negotiated