pub struct WebSocket<H> { /* private fields */ }Expand description
The trillium handler. See crate-level docs for example usage.
Implementations
Sourceimpl<T> WebSocket<JsonHandler<T>>where
T: JsonWebSocketHandler,
impl<T> WebSocket<JsonHandler<T>>where
T: JsonWebSocketHandler,
Sourcepub fn new_json(handler: T) -> Self
pub fn new_json(handler: T) -> Self
Build a new trillium WebSocket handler from the provided
JsonWebSocketHandler
Sourceimpl<H> WebSocket<H>where
H: WebSocketHandler,
impl<H> WebSocket<H>where
H: WebSocketHandler,
Sourcepub fn new(handler: H) -> Self
pub fn new(handler: H) -> Self
Build a new WebSocket with an async handler function that
receives a WebSocketConn
Sourcepub fn with_protocols(self, protocols: &[&str]) -> Self
pub fn with_protocols(self, protocols: &[&str]) -> Self
protocols is a sequence of known protocols. On successful handshake,
the returned response headers contain the first protocol in this list
which the server also knows.
Sourcepub fn with_protocol_config(self, config: WebSocketConfig) -> Self
pub fn with_protocol_config(self, config: WebSocketConfig) -> Self
configure the websocket protocol
Sourcepub fn required(self) -> Self
pub fn required(self) -> Self
configure this handler to halt and send back a 426 Upgrade Required if a websocket cannot be negotiated